/*************************************************
* OpenSSL Engine Header File                     *
* (C) 1999-2007 The Botan Project                *
*************************************************/

#ifndef BOTAN_EXT_ENGINE_OPENSSL_H__
#define BOTAN_EXT_ENGINE_OPENSSL_H__

#include <botan/engine.h>
#include <botan/mutex.h>

namespace Botan {

/*************************************************
* OpenSSL Engine                                 *
*************************************************/
class OpenSSL_Engine : public Engine
   {
   public:
      IF_Operation* if_op(const BigInt&, const BigInt&, const BigInt&,
                          const BigInt&, const BigInt&, const BigInt&,
                          const BigInt&, const BigInt&) const;
      DSA_Operation* dsa_op(const DL_Group&, const BigInt&,
                            const BigInt&) const;
      NR_Operation* nr_op(const DL_Group&, const BigInt&, const BigInt&) const;
      ELG_Operation* elg_op(const DL_Group&, const BigInt&,
                            const BigInt&) const;
      DH_Operation* dh_op(const DL_Group&, const BigInt&) const;

      Modular_Exponentiator* mod_exp(const BigInt&,
                                     Power_Mod::Usage_Hints) const;
   private:
      BlockCipher* find_block_cipher(const std::string&) const;
      StreamCipher* find_stream_cipher(const std::string&) const;
      HashFunction* find_hash(const std::string&) const;
   };

}

#endif
